@@ -4,6 +4,8 @@
 # License: MIT
 
 import cairo
+import generativepy.utils
+import numpy as np
 
 # Text align
 CENTER = 0
@@ -31,6 +33,8 @@ def setup(ctx, pixel_width, pixel_height, width=None, height=None, startx=0, sta
     '''
     Set up the context initial sclaling and background color
     :param ctx: The context
+    :param pixel_width: The device space width
+    :param pixel_height:  The device space width
     :param width: The user space width
     :param height:  The user space width
     :param startx: The x offset of the top left corner from the origin
@@ -71,11 +75,13 @@ def make_image(outfile, draw, width, height, channels=3):
     :param channels: 3 for rgb, 4 for rgba
     :return:
     '''
+    if outfile.lower().endswith('.png'):
+        outfile = outfile[:-4]
     fmt = cairo.FORMAT_ARGB32 if channels==4 else cairo.FORMAT_RGB24
     surface = cairo.ImageSurface(fmt, width, height)
     ctx = cairo.Context(surface)
     draw(ctx, width, height, 0, 1)
-    surface.write_to_png(outfile)
+    surface.write_to_png(outfile + '.png')
 
 
 def make_images(outfile, draw, width, height, count, channels=3):
@@ -87,8 +93,10 @@ def make_images(outfile, draw, width, height, count, channels=3):
     :param height: height in pixels, int
     :param count: number of frames to create
     :param channels: 3 for rgb, 4 for rgba
-    :return:
+    :return: a frame buffer
     '''
+    if outfile.lower().endswith('.png'):
+        outfile = outfile[:-4]
     for i in range(count):
         fmt = cairo.FORMAT_ARGB32 if channels==4 else cairo.FORMAT_RGB24
         surface = cairo.ImageSurface(fmt, width, height)
@@ -97,6 +105,50 @@ def make_images(outfile, draw, width, height, count, channels=3):
         surface.write_to_png(outfile + str(i).zfill(8) + '.png')
 
 
+def make_image_frames(draw, width, height, count, channels=3):
+    '''
+    Create a numpy frame file using cairo
+    :param draw: the draw function
+    :param width: width in pixels, int
+    :param height: height in pixels, int
+    :param count: number of frames to create
+    :param channels: 3 for rgb, 4 for rgba
+    :return: a lazy sequence of frame buffers
+    '''
+    fmt = cairo.FORMAT_ARGB32 if channels==4 else cairo.FORMAT_RGB24
+    for i in range(count):
+        surface = cairo.ImageSurface(fmt, width, height)
+        ctx = cairo.Context(surface)
+        draw(ctx, width, height, i, count)
+        buf = surface.get_data()
+        a = np.frombuffer(buf, np.uint8)
+        a.shape = (height, width, 4)
+        a = generativepy.utils.correct_pycairo_byte_order(a, channels)
+        yield a
+
+def make_image_frame(draw, width, height, channels=3):
+    '''
+    Create a numpy frame file using cairo
+    :param draw: the draw function
+    :param width: width in pixels, int
+    :param height: height in pixels, int
+    :param channels: 3 for rgb, 4 for rgba
+    :return:
+    '''
+    fmt = cairo.FORMAT_ARGB32 if channels==4 else cairo.FORMAT_RGB24
+    surface = cairo.ImageSurface(fmt, width, height)
+    ctx = cairo.Context(surface)
+    draw(ctx, width, height, 0, 1)
+    buf = surface.get_data()
+    a = np.frombuffer(buf, np.uint8)
+    a.shape = (height, width, 4)
+    if channels==3:
+        a[:, :, [0, 1, 2]] = a[:, :, [2, 1, 0]]
+    elif channels==4:
+        a[:, :, [0, 1, 2, 3]] = a[:, :, [2, 1, 0, 3]]
+    return a
+
+
 def make_svg(outfile, draw, width, height):
     '''
     Create an SVG file using cairo
@@ -106,7 +158,9 @@ def make_svg(outfile, draw, width, height):
     :param pixelSize: size in pixels tuple (x, y)
     :return:
     '''
-    surface = cairo.SVGSurface(outfile, width, height)
+    if outfile.lower().endswith('.svg'):
+        outfile = outfile[:-4]
+    surface = cairo.SVGSurface(outfile + '.svg', width, height)
     ctx = cairo.Context(surface)
     draw(ctx, width, height, 0, 1)
     ctx.show_page()
