@@ -9,6 +9,8 @@ from generativepy.drawing import EVEN_ODD, WINDING
 from generativepy.drawing import MITER, ROUND, BEVEL, BUTT, SQUARE
 from generativepy.color import Color
 
+# DEPRECATED - replace with easy_vector, then remove from utils
+from generativepy.utils import vector_unit, vector_a_b
 
 class Shape():
 
@@ -50,11 +52,14 @@ class Shape():
         self.ctx.fill_preserve()
         return self
 
-    def stroke(self, color=Color(0), line_width=1, dash=[], cap=SQUARE, join=MITER, miter_limit=None):
+    def stroke(self, color=Color(0), line_width=1, dash=None, cap=SQUARE, join=MITER, miter_limit=None):
         if not self.added:
             self.add()
             self.added = True
 
+        if not dash:
+            dash = []
+
         self.ctx.set_source_rgba(*color)
 
         self.ctx.set_line_width(line_width)
@@ -81,7 +86,7 @@ class Shape():
         self.ctx.stroke_preserve()
         return self
 
-    def fill_stroke(self, fill_color, stroke_colour, line_width=1):
+    def fill_stroke(self, fill_color, stroke_color, line_width=1):
         self.fill(fill_color)
         self.stroke(stroke_color, line_width)
         return self
@@ -302,6 +307,19 @@ class Text(Shape):
         self._offset = (x, y)
         return self
 
+    def offset_angle(self, angle, distance):
+        x = distance*math.cos(angle)
+        y = distance*math.sin(angle)
+        self._offset = (x, y)
+        return self
+
+    def offset_towards(self, point, distance):
+        unit = vector_unit(vector_a_b(self.position, point))
+        x = distance*unit[0]
+        y = distance*unit[1]
+        self._offset = (x, y)
+        return self
+
 
 
 def text(ctx, txt, x, y, font=None, size=None, color=None, alignx=LEFT, aligny=BASELINE, flip=False):
@@ -324,7 +342,7 @@ def text(ctx, txt, x, y, font=None, size=None, color=None, alignx=LEFT, aligny=B
     if font:
         shape = shape.font(font)
     if size:
-        shape = shape.flip()
+        shape = shape.size(size)
     if flip:
         shape = shape.flip()
 
@@ -707,3 +725,32 @@ def arrowhead(ctx, a, b, length=4):
     ctx.new_path()
     draw(b[0], b[1], (-vector[0] + pvector[0]) * length / 2, (-vector[1] + pvector[1]) * length / 2,
          (-vector[0] - pvector[0]) * length / 2, (-vector[1] - pvector[1]) * length / 2)
+
+class Image():
+
+    def __init__(self, ctx):
+        self.ctx = ctx
+        self.filename = ''
+        self.position = (0, 0)
+        self.scale_factor = 1
+
+    def of_file_position(self, filename, position):
+        self.filename = filename
+        self.position = position
+        return self
+
+    def scale(self, scale_factor):
+        self.scale_factor = scale_factor
+        return self
+
+    def paint(self):
+        image = cairo.ImageSurface.create_from_png(self.filename)
+        self.ctx.save()
+        self.ctx.translate(*self.position)
+        self.ctx.scale(self.scale_factor, self.scale_factor)
+        pattern = cairo.SurfacePattern(image)
+        self.ctx.set_source(pattern)
+        self.ctx.rectangle(0, 0, image.get_width(), image.get_height())
+        self.ctx.fill()
+        self.ctx.restore()
+        return self
